"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var solver = require("javascript-lp-solver/src/solver");
function isSubset(a, b) {
    var a2 = a.slice();
    for (var _i = 0, b_1 = b; _i < b_1.length; _i++) {
        var n = b_1[_i];
        var i = a2.indexOf(n);
        if (i !== -1) {
            a2.splice(i, 1);
            if (a2.length === 0) {
                return true;
            }
        }
    }
    return a2.length === 0;
}
function howManyWays(size, cuts, state) {
    if (state === void 0) { state = []; }
    return _.uniqWith(_.flatten(cuts.map(function (cut) {
        var remainder = size - cut;
        if (remainder >= 0) {
            return howManyWays(remainder, cuts, state.concat([cut]));
        }
        else {
            return [state];
        }
    })), function (x, y) { return isSubset(x, y) || isSubset(y, x); });
}
exports.howManyWays = howManyWays;
function howToCutBoards1D(stockSize, requiredCuts) {
    var cutSizes = requiredCuts.map(function (_a) {
        var size = _a.size;
        return size;
    });
    var waysOfCutting = howManyWays(stockSize, cutSizes);
    var stockVersions = waysOfCutting.map(function (way) {
        var stockCut = {};
        for (var _i = 0, cutSizes_1 = cutSizes; _i < cutSizes_1.length; _i++) {
            var cut = cutSizes_1[_i];
            stockCut["cut" + cut] = 0;
        }
        for (var _a = 0, way_1 = way; _a < way_1.length; _a++) {
            var cut = way_1[_a];
            stockCut["cut" + cut] = stockCut["cut" + cut] + 1;
        }
        return stockCut;
    });
    var variables = stockVersions
        .map(function (cut, index) {
        var _a;
        return (_a = {}, _a["version" + index] = __assign({}, cut, { count: 1 }), _a);
    })
        .reduce(function (acc, next) { return (__assign({}, acc, next)); });
    var ints = stockVersions
        .map(function (cut, index) {
        var _a;
        return (_a = {}, _a["version" + index] = 1, _a);
    })
        .reduce(function (acc, next) { return (__assign({}, acc, next)); });
    var constraints = requiredCuts
        .map(function (_a) {
        var size = _a.size, count = _a.count;
        var _b;
        return (_b = {}, _b["cut" + size] = { min: count }, _b);
    })
        .reduce(function (acc, next) { return (__assign({}, acc, next)); });
    var model = {
        optimize: "count",
        opType: "min",
        variables: variables,
        int: ints,
        constraints: constraints,
    };
    var results = solver.Solve(model);
    if (!results.feasible) {
        throw new Error("Didn't work");
    }
    var resultCuts = [];
    for (var i = 0; i < waysOfCutting.length; i++) {
        var number = results["version" + i];
        if (number !== undefined && number > 0) {
            resultCuts.push({
                count: Math.ceil(number),
                decimal: number,
                cuts: waysOfCutting[i],
            });
        }
    }
    return resultCuts;
}
exports.howToCutBoards1D = howToCutBoards1D;
//# sourceMappingURL=index.js.map